#include <stdio.h>
#include "gidml_io.h"

// This is a basic test example just to show how the gidml_io library
// can be used to read and write a custom example
// Compile with:
//   g++ -o test_io test_io.cc -std=c++0x -I. -L../lib/ -lgidml_io -lhdf5_hl -lhdf5

int Read_GiDML_IO_AndWriteGiDMesh( const char *filename );

int main() {
  printf( "Version of gidml_io library: %s\n", GiDML_IO_GetVersion() );
  printf( "Vertion of gidml_io format:  %s\n", GiDML_IO_GetFormatVersion() );
  printf( "   Supported element types: " );
  for ( int i = 0; i < GetNumElemTypes() - 1; i++ ) {
    printf( "%s, ", GetElementTypeName( ( ElemType )i ) );
  }
  printf( "%s", GetElementTypeName( ( ElemType )( GetNumElemTypes() - 1 ) ) );
  // write out an example:
  GiDOutput_Handle gid_out_hnd = GiDML_IO_NewGiDOutputHandle();
  GiDML_IO_SetMeshDimension( gid_out_hnd, 3 );
  double nodes_coords[] = {
    0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0
  }; // nodes of a square on plane z = 0
  GiDML_IO_SetNodesCoords( gid_out_hnd, 4, nodes_coords );
  int elements[] = { 0, 1, 2, 3 }; // single square
  const int nnode = 4;
  GiDML_IO_SetElements( gid_out_hnd, 1, elements, GID_QUADRILATERAL_ELEMENT, nnode );
  int fail = GiDML_IO_WriteGiDOutput( "example_out.gidml", gid_out_hnd );
  printf( "WriteGiDOutput return = %d\n", fail );
  GiDML_IO_DeleteGiDOutputHandle( gid_out_hnd );

  // test reading the .gidml file and export a .msh GiD mesh ASCII file 
  // (which can be read by all GiD versions)
  fail = Read_GiDML_IO_AndWriteGiDMesh( "example_out.gidml" );
  return fail;
}

int Read_GiDML_IO_AndWriteGiDMesh( const char *filename ) {
  // function to test read from gidml_io format (HDf5-based) and write in gid mesh ASCII format.
  GiDOutput_Handle goutput_hdl = NULL;
  int fail = GiDML_IO_ReadGiDOutput( filename, goutput_hdl );
  if ( goutput_hdl ) {
    fail = GiDML_IO_WriteMeshInGiDFormat( "example_out.msh", goutput_hdl );
    GiDML_IO_DeleteGiDOutputHandle( goutput_hdl );
  }
  return fail;
}
